import 'dart:async';
import 'dart:convert';
import 'dart:typed_data';
import 'package:flutter/foundation.dart';
import 'package:mqtt_client/mqtt_client.dart';
import './config.dart';
import 'package:http/http.dart' as http;
import './socketManager/socketManager.dart';
import 'package:crypto/crypto.dart' as crypto;

typedef void OnConnectCallBack(bool isconnected);
typedef void OnMessageCallBack(String msg);
typedef void OnSubscribeCallBack(String lastsubscribe);
typedef void OnPresenceCallBack(String msg);
String lastTopic;

class Emitter {
  static final Emitter _instance = Emitter._privateConstructor();
  static Emitter get instance => _instance;
  Emitter._privateConstructor();
  String user;
  String myId = "";
  dynamic _mqtt;
  OnConnectCallBack onConnect;
  OnMessageCallBack onMessage;
  OnSubscribeCallBack onsubscribe;
  OnPresenceCallBack onPresence;
  bool _logging = false;
  Map<int, Completer> _completers = new Map<int, Completer>();
  Map<String, dynamic> headers = {};
  Map<String, List<Uint8List>> chunks = {};

  Future<dynamic> callAPI(auth_token, project_id) async {
    final datarequest = {"auth_token": auth_token, "project_id": project_id};
    print("The host:$host");
    final response =
        await http.post(Uri.parse('$host'), body: json.encode(datarequest));
    print("this is response of Api call ${json.decode(response.body)}");
    if (response.statusCode == 200) {
      print("${response.statusCode}");
      return json.decode(response.body);
    } else {
      throw Exception("Failed to Load Data");
    }
  }

  Future<bool> connect(
      {String clientId,
      bool reconnectivity,
      String refID,
      String authorization_token,
      String auth_token,
      String project_id}) async {
    var data = await callAPI(auth_token, project_id);
    // print("The data is: ${data["messaging_server"].host.toString()}");
    //var splittted = data["messaging_server_map"]["complete_address"].toString().split(":");
    // var host = splittted[0] + ":" + splittted[1];
    // print("The host of mqtt:$host....port:${splittted[2]}");

    var host = "wss://" + data["messaging_server_map"]["host"];
    var port = int.parse(data["messaging_server_map"]["port"]);
    bool secure = false;
    int keepalive = 5;
    // bool logging = true;
    bool useWebSocket = true;
    String brokerUrl = host;
    print("here ###################");
    //_mqtt.port = port;
    _mqtt = await SocketManager.instance.connect(brokerUrl);

    _mqtt.port = port;
    if (!kIsWeb) {
      _mqtt.secure = secure;
      _mqtt.useWebSocket = useWebSocket;
      _mqtt.useAlternateWebSocketImplementation = false;
    }
    _mqtt.logging(on: false);
    _mqtt.clientIdentifier = clientId;
    _mqtt.keepAlivePeriod = keepalive;
    _mqtt.onDisconnected = _onDisconnected;
    _mqtt.onConnected = _onConnected;
    _mqtt.onSubscribed = _onSubscribed;
    _mqtt.onUnsubscribed = _onUnsubscribed;
    _mqtt.autoReconnect = reconnectivity;
    _mqtt.onSubscribeFail = _onSubscribeFail;
    // _mqtt.securityContext=
    // _logging = logging;
    // final MqttConnectMessage connMess = MqttConnectMessage()
    //     //   .keepAliveFor(keepalive)
    //     .startClean()
    //     .withClientIdentifier(clientId)
    //     .authenticateAs(refID, authorization_token)
    //     .withWillQos(MqttQos.atLeastOnce);
    // _mqtt.connectionMessage = connMess;

    try {
      print("before");
      await _mqtt.connect(refID, authorization_token);
      _mqtt.keepAlivePeriod = keepalive;
      _mqtt.autoReconnect = reconnectivity;
    } on Exception catch (e) {
      print("This is Exception: $e");
      _mqtt.disconnect();
      return false;
    }

    _mqtt.updates.listen((List<MqttReceivedMessage<MqttMessage>> c) async {
      final MqttPublishMessage recMess = c[0].payload;
      final String topic = c[0].topic;
      //print("this is recMess ${c[0].payload}");
      var msg =
          MqttPublishPayload.bytesToStringAsString(recMess.payload.message);
      print("mesg is before $msg");

      print("topic: $topic");
      var id;
      if (topic.startsWith("emitter/me/")) {
        print("hereee");
        //id= json.decode(utf8.decode(msg));
        id = json.decode(msg);
        print("id is $id");
        if (id != null) {
          print("hereee if");
          if (id["id"] != null) {
            var me = id["id"];
            myId = me;
            id = myId;
            print(" IDDdd $myId");
          }
        }
      } else if (topic.startsWith("emitter/presence/")) {
        print("hereee else if");
        if (msg != null) {
          onPresence.call(msg);
        }
      } else if (!topic.startsWith("emitter/me/")) {
        print("mesg is-- $msg");
        final message = json.decode((msg));
        print("this is decoded message $message");
        if (message["type"] == MediaType.audio ||
            message["type"] == MediaType.video ||
            message["type"] == MediaType.image ||
            message["type"] == MediaType.file) {
          chunkstoFile(message);
        } else {
//            List<int> bytes = utf8.encode(message["content"]);
// String value = utf8.decode(bytes);
//   print('bytes: $bytes');
//   print('value: $value');
          print("i am in elseeee");
          print("message string is ${json.decode(msg)}");
          var jsonMessage = json.decode(msg);
          if (jsonMessage["content"] != null) {
            jsonMessage["content"] =
                utf8.decode(jsonMessage["content"].toString().codeUnits);
            print("this is with emojies  ${jsonMessage["content"]} ");
            onMessage.call(json.encode(jsonMessage));
          } else {
            onMessage.call(msg);
          }
        }
      }
    });

    var me = await getMe();
    myId = me["id"];

    return true;
  }

  chunkstoFile(Map<String, dynamic> message) {
    print("this is meedia message ${message["date"]}");
    if (message["totalPacket"] != null) {
      print("this is heeader $message");
      chunks.addAll({message["headerId"]: []});
      headers.addAll({message["headerId"]: message});
      // header = message;
    } else {
      if (headers[message["headerId"]]["totalPacket"] - 1 ==
          chunks[message["headerId"]].length) {
        chunks[message["headerId"]]
            .insert(--message["packetNo"], base64.decode(message["content"]));
        List<int> fromchunks =
            chunks[message["headerId"]].expand((element) => element).toList();
        Uint8List listFromChunks = Uint8List.fromList(fromchunks);
        print("this is listofchunks ${listFromChunks.length}");
        message.remove("content");
        message.remove("packetNo");
        message.addAll({
          "content": base64.encode(listFromChunks),
          "id": headers[message["headerId"]]["id"],
          "fileExtension": headers[message["headerId"]]["fileExtension"],
          "date": headers[message["headerId"]]["date"]
        });
        print("this is message after content reemove $message");
        onMessage.call(json.encode(message));
        headers.remove(message["headerId"]);
        chunks.remove(message["headerId"]);
      } else {
        chunks[message["headerId"]]
            .insert(--message["packetNo"], base64.decode(message["content"]));
      }
    }
  }

  generateMd5(String groupkey) {
    var input = DateTime.now().millisecondsSinceEpoch.toString() + groupkey;
    var content = crypto.md5.convert(utf8.encode(input)).toString();
    print("the input value in MD5 format: $content");
    return content;
  }

// create  list of chunks of bytes
  List<Uint8List> byteToChunks(Uint8List largeByteArray) {
    // create a list to keep the portions
    List<Uint8List> byteArrayPortions = [];

    int sizePerPortion = chunkSize;
    int offset = 0;
    for (int i = 0; i <= (largeByteArray.length / sizePerPortion); i++) {
      int end = sizePerPortion * (i + 1);
      if (end >= largeByteArray.length) {
        end = largeByteArray.length;
      }
      Uint8List portion = largeByteArray.sublist(offset, end);
      offset = end;
      byteArrayPortions.add(portion);
    }
    return byteArrayPortions;
  }

  //****PUBLISH MESSAGE ****//
  Future<int> publish(
    String key,
    String channel,
    Map<String, dynamic> message,
  ) async {
    // if (message["type"] != "RECEIPTS")
    print("publish message");
    // message["id"] = generateMd5(key);
    // divide image to chunks
    if (message["type"] == MediaType.audio ||
        message["type"] == MediaType.video ||
        message["type"] == MediaType.image ||
        message["type"] == MediaType.file) {
      List<Uint8List> listofChunks =
          byteToChunks(base64.decode(message["content"]));

      Map<String, dynamic> header = {
        "headerId": generateMd5(key),
        "totalPacket": listofChunks.length,
        "size": base64.decode(message["content"]).length,
        "topic": message["to"],
      };
      message.remove("content");
      message.addAll(header);
      print("this is header of file $message");
      publishMessage(key, channel, message);

      for (int i = 1; i <= listofChunks.length; i++) {
        await Future.delayed(Duration(microseconds: 0), () {
          // Do something
          print("publish message in loop $i");
          Map<String, dynamic> filePacket = {
            "messageId": message["id"],
            "packetNo": i,
            "topic": message["to"],
            "content": base64.encode(listofChunks[i - 1]),
          };
          message.remove("totalPacket");
          message.remove("size");
          message.addAll(filePacket);
          print("this is file packet$message");

          publishMessage(key, channel, message);
        });
      }
    } else {
      publishMessage(key, channel, message);
    }

    // print("this is publised message $message");
    // MqttQos qos = MqttQos.atLeastOnce;
    // bool retain = false;
    // // var options = new Map<String, String>();
    // // if (!me) options["me"] = "0";
    // // if (ttl > 0) options["ttl"] = ttl.toString();
    // var topic = _formatChannel(key, channel, null);
    // // var chatid = generateMd5(key);
    // // print("chat id: $chatid");
    // print("This is before publish");
    // final builder = MqttClientPayloadBuilder();
    // builder.addString(jsonEncode(message));
    // return _mqtt.publishMessage(topic, qos, builder.payload, retain: retain);
  }

  publishMessage(
    String key,
    String channel,
    Map<String, dynamic> message,
  ) {
    print("this is publised message ${message["type"]}");
    MqttQos qos = MqttQos.atLeastOnce;
    bool retain = false;
    // var options = new Map<String, String>();
    // if (!me) options["me"] = "0";
    // if (ttl > 0) options["ttl"] = ttl.toString();
    var topic = _formatChannel(key, channel, null);
    // var chatid = generateMd5(key);
    // print("chat id: $chatid");

    print("This is before publish ${jsonEncode(message)}");
    print("this is content of message before ${message["content"]}");
    try {
      //Uint8Buffer _payload;
      final builder = MqttClientPayloadBuilder();
      // List<int> bytes = jsonEncode(message).codeUnits;
      // // utf8.encode(jsonEncode(message));
      // Uint8List list = Uint8List.fromList(bytes);
      // print("these are bytes $bytes");

      // builder.addString(String.fromCharCode(list));
      // const encoder = Utf8Encoder();
      // Uint8List mesg = encoder.convert(jsonEncode(message));
      // builder.addBuffer(mesg);

      // builder.addUTF8String(utf8.encode(jsonEncode(message)).toString());
      // print(
      //     "this is content of message after ${(utf8.encode(jsonEncode(message)).toString())}");
      builder.addUTF8String(jsonEncode(message));

      _mqtt.publishMessage(topic, qos, builder.payload, retain: retain);
    } catch (e) {
      print("i am here in catch");
      print("this is error $e");
    }
  }

  //****GENERATING UNIQUE ID FOR EACH MESSAGE ****//
  // generateMd5(String groupkey) {
  //   var input = DateTime.now().millisecondsSinceEpoch.toString() + groupkey;
  //   var content = crypto.md5.convert(utf8.encode(input)).toString();
  //   print("the input value in MD5 format: $content");
  //   return content;
  // }

  //****SUBSCRIBE MESSAGE ****//
  Future<Subscription> subscribe(
    String key,
    String channel, {
    int last = 0,
  }) async {
    // var options = new Map<String, String>();
    // if (last > 0) options["last"] = last.toString();
    var topic = _formatChannel(key, channel, null);
    return _mqtt.subscribe(topic, MqttQos.atLeastOnce);
  }

  /*
  * Unsubscribes from a particular channel.
  */
  void unsubscribe(String key, String channel) {
    var topic = _formatChannel(key, channel, null);
    _mqtt.unsubscribe(topic);
  }

  //****MAKE TOPIC FROM CHANNEL AND KEY****//
  _formatChannel(String key, String channel, Map<String, String> options) {
    var formatted = channel;
    // Prefix with the key if any
    if (key != null && key.length > 0)
      formatted = key.endsWith("/") ? key + channel : key + "/" + channel;
    // Add trailing slash
    if (!formatted.endsWith("/")) formatted += "/";
    // Add options
    if (options != null && options.length > 0) {
      formatted += "?";
      options.forEach((key, value) {
        formatted += key + "=" + value + "&";
      });
    }
    if (formatted.endsWith("&"))
      formatted = formatted.substring(0, formatted.length - 1);
    // We're done compiling the channel name
    return formatted;
  }

  Future<dynamic> getMe({int timeout = 5000}) async {
    var response = await _executeAsync("emitter/me/", "");
    myId = response;
  }

  void disconnect() {
    _mqtt.disconnect();
  }

  Future<dynamic> _executeAsync(String request, String payload) {
    Completer c = new Completer<dynamic>();
    final builder = MqttClientPayloadBuilder();
    builder.addString(jsonEncode(payload));
    int id =
        _mqtt.publishMessage(request, MqttQos.atLeastOnce, builder.payload);
    _completers[id] = c;
    return c.future;
  }

  //*******GET PRESENCE*******//
  Future<dynamic> getPresence(String key, String channel,
      {int timeout = 5000}) async {
    var request = {
      "key": key,
      "channel": channel,
      "status": true,
      "changes": false
    };
    var response =
        await _executeAsync("emitter/presence/", jsonEncode(request));
    return response;
  }

  int subscribePresence(String key, String channel, bool changes, bool status) {
    var request = {
      "key": key,
      "channel": channel,
      "status": changes,
      "changes": status
    };
    final builder = MqttClientPayloadBuilder();
    builder.addUTF8String(jsonEncode(request));
    return _mqtt.publishMessage(
        "emitter/presence/", MqttQos.atLeastOnce, builder.payload);
  }

  //****_ONDISCONNECTED CALLBACK****//
  _onDisconnected() {
    print('EMITTER::Disconnected');
    onConnect.call(false);
  }

  //****_ONCONNECTED CALLBACK****//
  _onConnected() async {
    print('EMITTER::Connected');
    //Future.delayed(Duration(seconds: 5), () {
    onConnect.call(true);
    //  });
  }

  //****_ONSUBSCRIBED CALLBACK****//
  _onSubscribed(String topic) async {
    print('EMITTER::Subscription confirmed for topic $topic');

    onsubscribe.call(topic);
  }

  //****_ONUNSUBSCRIBED CALLBACK****//
  _onUnsubscribed(String topic) async {
    print('EMITTER::Unsubscription confirmed for topic $topic');
  }

  //****_ONSUBSCRIPTION FAILED CALLBACK****//
  _onSubscribeFail(String topic) {
    _log('EMITTER::Subscription failed for topic $topic');
  }

  _log(Object message) {
    if (_logging) print(message);
  }

  _pong() {
    _log('EMITTER::Pong');
  }
}
